/* mprl.c -- Main program file
   Copyright (C) 2003, 2004
   Arturo "Buanzo" Busleiman <buanzo@buanzo.com.ar> <buanzo@linux.org.ar>
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pwd.h>
#include <sys/types.h>
#include <errno.h>
#include "misc.h"
#include "parser.h"
#include "buennoc.h"

int
main (int argc, char **argv)
{
  char *userdata, *validdata, *username, *hostname, *protocol, *strport,
    *whichp;
  struct passwd *pwd;
  short onlylocal = 0;
  short silent = 0;
  struct proto_struct *proto_params;
  int i = 0;
  int error_code = 0;

#ifdef DEBUG
  short wwi = 0;
  printf ("argc = %d\n", argc);
  for (wwi = 0; wwi < argc; wwi++)
    printf ("argv[%d] = %s\n", wwi, argv[wwi]);
#endif

  if ((argc == 4) && (strcmp (argv[3], "-q") == 0))
    silent = 1;
  if (!silent)
    print_mprl_version ();

  pwd = getpwnam ("mprl");

  if (pwd == NULL)
    {
      if (errno == ENOMEM)
	{
	  perror ("mprl");
	  sleep (3);
	  exit (2);
	}
      else
	{
	  printf
	    ("\n\033[1;34mBad Installation! User/Group mprl not added. Check documentation.\n\033[0;37mOnly allowing LOCAL logins.\n\n");
	  onlylocal = 1;
	}
    }
  if (parse_mprl_conf (NULL, &error_code) == NULL)
    onlylocal = 1;
  userdata = NULL;


  if (argc >= 3)
    {
      userdata = argv[2];
    }
  else
    {
      printf ("Syntax error.\n");
      sleep (3);
      exit (255);
    }

  validdata = (char *) strtok (userdata, "\n");

  userdata = (char *) strtok (validdata, "@");
  hostname = (char *) strtok (NULL, "@");

  protocol = (char *) strtok (userdata, ":");
  username = (char *) strtok (NULL, ":");

  if (username == NULL)
    {
      username = protocol;
      protocol = NULL;
    }

  userdata = hostname;
  hostname = (char *) strtok (userdata, ":");
  strport = (char *) strtok (NULL, ":");

/* now set default value to each NULL variable */

/* Special case: username is null -> stop execution  */

  if (username == NULL)
    {
      printf ("\nNo username given. Bye!\n");
      fflush (stdout);
      sleep (3);
      exit (1);
    }

/* default scanning: if no hostname and no proto is provided, 
assume proto = login (sets hostname -> localhost), and if no proto is 
provided BUT a hostname is, then assume proto=ssh. 
NOTE THE ADDED ':' AT THE END OF THE PROTOCOL'S NAME */

  if ((protocol == NULL) && (hostname != NULL))
    protocol = "ssh";
  if (hostname == NULL)
    hostname = "localhost";
  if ((hostname == "localhost") && (protocol == NULL))
    protocol = "login";


  if (!silent)
    printf ("\n\033[1;32mLogin URL: \033[0;37m%s:%s@%s:%s\n\n", protocol,
	    username, hostname, strport ? strport : "\b ");

  if (strcmp (protocol, "login") == 0)
    {
      execl ("/bin/login", "/bin/login", "--", username, NULL);
      print_execfamily_error ();
    }

  if (onlylocal)
    {
      printf
	("\nSorry, only local logins allowed. Maybe /etc/mprl.conf does not exist, or is\nunreadable? Check mprl's documentation.\n\n");
      sleep (3);
      exit (255);
    }

  if (strcmp (protocol, "login:") != 0)
    {
      setgid (pwd->pw_gid);
      setuid (pwd->pw_uid);
    }

  whichp = buennoc (strlen (protocol) + 4,0);
  strncpy (whichp, protocol, strlen (protocol) + 1);
  strncat (whichp, ":", 1);
  proto_params = parse_mprl_conf (whichp, &error_code);
  if (proto_params == NULL)
    {
      switch (error_code)
	{
	case ERR_PARSER_OPEN_CONFIG:
	  {
	    printf ("\n\033[1;32mCannot open mprl.conf file.\033[0;37m\n");
	    break;
	  }
	case ERR_PARSER_CANNOT_ALLOC:
	  {
	    printf
	      ("\n\033[1;32mCannot allocate memory for ldef struct.\033[0;37m\n");
	    break;
	  }
	case ERR_PARSER_NOT_PROTOCOL:
	  {
	    printf ("\n\033[1;32mUnknown Protocol.\033[0;37m\n");
	    break;
	  }
	}
    }
  else
    {
      for (i = 0; i < proto_params->n_args; i++)
	{
	  if (strcmp (proto_params->args[i], "%USER") == 0)
	    {
	      free (proto_params->args[i]);
	      proto_params->args[i] = username;
	    }
	  else if (strcmp (proto_params->args[i], "%PASS") == 0)
	    {
	      printf ("\nPassword request not implemented\n");
	      sleep (5);
	      exit (255);
	    }
	  else if (strcmp (proto_params->args[i], "%HOST") == 0)
	    {
	      free (proto_params->args[i]);
	      proto_params->args[i] = hostname;
	    }
	  else if (strcmp (proto_params->args[i], "%PORT") == 0)
	    {
	      free (proto_params->args[i]);
	      if (strport != NULL)
		proto_params->args[i] = strport;
	      else
		{
		  proto_params->args[i] =
		    buennoc (sizeof (*proto_params->default_port) + 1,0);
		  proto_params->args[i] = proto_params->default_port;
		}
	    }
	}

#ifdef DEBUG
      printf ("\nAfter variable replacing..\n");
      for (i = 0; i < proto_params->n_args; i++)
	printf ("[%d] = '%s'\n", i, proto_params->args[i]);
#endif

/* Execution... parser.c adds a final NULL args member, needed by exec() */
      execv (proto_params->args[0], proto_params->args);

    }

  exit (0);
}
